"use strict";
/// <reference path="pal.WebOS.ts"/>
/// <reference path="main.ts"/>
/// <reference path="SourcePatcher.js"/>
/// <reference path="moment.min.js"/>
var baseDirectory = "downloads";
var cacheRetainDays = 7;
;
var DownloadNetworkType;
(function (DownloadNetworkType) {
    DownloadNetworkType[DownloadNetworkType["CELLULAR"] = 0] = "CELLULAR";
    DownloadNetworkType[DownloadNetworkType["WIFI"] = 1] = "WIFI";
    DownloadNetworkType[DownloadNetworkType["ALL"] = 2] = "ALL";
})(DownloadNetworkType || (DownloadNetworkType = {}));
;
var korbytStaticContent = [
    '/fonts/',
    '/images/',
    '/img/',
    '/jClocksGMT/',
    '/builder/1.5/'
];
var ResourceManager = /** @class */ (function () {
    function ResourceManager() {
    }
    ResourceManager.initializer = function () {
        ResourceManager.downloadPossible = true;
        // _klogger.log(`downloadPossible: ${ResourceManager.downloadPossible}`);
        ResourceManager.dlPending = [];
        ResourceManager.staticCache = {};
        ResourceManager.isDownloading = false;
        ResourceManager.cacheCleanTimer = null;
        // cannot name download path file://internal/cache
        ResourceManager.FullDownloadPath = 'file://internal';
        ResourceManager.cacheRoot = 'file://internal';
        ResourceManager.storage = new Storage;
        console.log("Success intializing the cache path is " + ResourceManager.fullDownloadPath);
    };
    Object.defineProperty(ResourceManager, "FullDownloadPath", {
        get: function () {
            return ResourceManager.fullDownloadPath;
        },
        set: function (path) {
            if (ResourceManager.fullDownloadPath !== path) {
                ResourceManager.fullDownloadPath = path;
                localStorage.setItem('fullDownloadPath', ResourceManager.fullDownloadPath);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResourceManager, "PlayerSplashFullPath", {
        get: function () {
            return ResourceManager.playerSplashFullPath;
        },
        set: function (path) {
            if (ResourceManager.playerSplashFullPath !== path) {
                ResourceManager.playerSplashFullPath = path;
                localStorage.setItem('playerSplashFullPath', ResourceManager.playerSplashFullPath);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ResourceManager, "IsDownloading", {
        get: function () {
            return ResourceManager.isDownloading;
        },
        enumerable: true,
        configurable: true
    });
    ResourceManager.isStaticContent = function (fn) {
        if (typeof fn === 'string') {
            for (var i_1 = 0; i_1 < korbytStaticContent.length; i_1++) {
                if (fn.startsWith(korbytStaticContent[i_1])) {
                    return true;
                }
            }
        }
        return false;
    };
    ResourceManager.getResources = function (cleanCache, resources, showProgress, callback) {
        // console.log("START OF GET RESOURCES FUNCTION");
        // console.log(ResourceManager.isDownloading);
        if (ResourceManager.isDownloading) {
            ResourceManager.dlPending = ResourceManager.dlPending.concat(resources.slice(0));
            return callback();
        }
        else {
            ResourceManager.isDownloading = true;
            ResourceManager.startTime = new Date();
            clearTimeout(ResourceManager.cacheCleanTimer);
            // console.log(resources);
            try {
                if (Array.isArray(resources) && resources.length > 0) {
                    ResourceManager.countFilesDownloaded = 0;
                    // always include the manifest files in the keeper list.
                    ResourceManager.resourceKeeper = ResourceManager.manifestList.slice();
                    ResourceManager.cleanCacheProcess = cleanCache;
                    ResourceManager.dlPending = ResourceManager.dlPending.concat(resources.slice(0));
                    // console.log(`GET RESOURCES DL PENDING: ${ResourceManager.dlPending}`);
                    // console.log(`DL Pending: ${ResourceManager.dlPending}`);
                    ResourceManager.finalCallback = callback;
                    return ResourceManager.downloadNext();
                }
            }
            catch (err) {
                _klogger.error(err);
                return callback(err);
            }
        }
    };
    ResourceManager.deleteDownloadedFile = function (filePath, callback) {
        var options = { file: filePath };
        return ResourceManager.storage.removeFile(function () {
            _klogger.info("Successfully Removed: " + filePath);
        }, function (error) {
            console.log("Failed to Remove: " + filePath);
            _klogger.error(error);
        }, options);
    };
    ResourceManager.start = function (tenantId, showProgress, callback) {
        _klogger.info("BEGINNING TO DOWNLOAD THE PRT");
        try {
            clearTimeout(ResourceManager.cacheCleanTimer);
            ResourceManager.dlPending = [];
            ResourceManager.manifestList = [];
            if (ResourceManager.downloadPossible) {
                ResourceManager.forceDownloads = true;
                return ResourceManager.getResources(false, ['/prt/v17/manifest.json'], showProgress, function () {
                    try {
                        ResourceManager.forceDownloads = false;
                        var options = {
                            path: ResourceManager.cacheRoot + "/prt/v17/manifest.json"
                        };
                        ResourceManager.storage.readFile(function (res) {
                            ResourceManager.dlPending = [];
                            var resources = res.data;
                            resources = JSON.parse(resources);
                            for (var i = 0; i < resources.length; i++) {
                                resources[i] = '/prt/v17/' + resources[i];
                            }
                            if (main.pal.CheckInResults.instructions) {
                                var checkInInstructions = main.pal.CheckInResults.instructions.resources;
                                resources = checkInInstructions.concat(resources);
                                ResourceManager.manifestList = resources.slice(0);
                            }
                            return ResourceManager.getResources(false, resources, showProgress, callback);
                        }, function (error) {
                            console.log("Errror Code: " + error.errorCode + "; with message: " + error.errorMessage);
                            _klogger.error(error);
                        }, options);
                    }
                    catch (err) {
                        _klogger.error(err);
                        return callback(err);
                    }
                });
            }
        }
        catch (err) {
            _klogger.error(err);
            return callback(err);
        }
    };
    ResourceManager.getFilename = function (path) {
        try {
            return path.split('\\').pop().split('/').pop();
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.getDirectory = function (path) {
        try {
            return path.substring(0, path.lastIndexOf("/"));
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.downloadCacheFile = function (dlResource) {
        try {
            if (!dlResource.serverUrl) {
                throw new Error('downloadCacheFile(): dlResource.serverUrl is not defined');
            }
            if (dlResource.serverPath.startsWith('/embed/') || dlResource.serverPath.startsWith('/message/') || dlResource.serverPath.startsWith('/urgent/')) {
                dlResource.serverPath = dlResource.serverPath + '.html';
                dlResource.localFullDirectory = dlResource.localFullDirectory + '.html';
                dlResource.fileName = dlResource.fileName + '.html';
            }
            if (dlResource.serverPath.startsWith('/embed/') || dlResource.serverPath.startsWith('/message/') || dlResource.serverPath.startsWith('/urgent/')) {
                var xhr = new XMLHttpRequest();
                xhr.open('GET', dlResource.serverUrl);
                xhr.responseType = "arraybuffer";
                xhr.setRequestHeader('Accept', '*/*');
                xhr.onreadystatechange = function () {
                    if (xhr.readyState == 4) {
                        if (xhr.status === 200 || xhr.status === 206) {
                            var buf = xhr.response;
                            var responseBuffer = new Int8Array(buf);
                            var input = {
                                data: buf,
                                path: "" + ResourceManager.cacheRoot + dlResource.localFullDirectory,
                                offset: 0,
                                encoding: "binary",
                                numberOfChunks: Math.ceil(responseBuffer.length / 10240)
                            };
                            var idx_1 = 0;
                            var existsOptions = { path: "" + ResourceManager.cacheRoot + dlResource.localFullDirectory };
                            return ResourceManager.storage.exists(function (res) {
                                if (res.exists != true) {
                                    return ResourceManager.htmlChunker(input, idx_1);
                                }
                                else {
                                    var removeOption = { file: "" + ResourceManager.cacheRoot + dlResource.localFullDirectory };
                                    return ResourceManager.storage.removeFile(function () {
                                        return ResourceManager.htmlChunker(input, idx_1);
                                    }, function (error) {
                                        console.log("ERROR " + JSON.stringify(error));
                                        _klogger.error(error);
                                    }, removeOption);
                                }
                            }, function (error) {
                                console.log("ERROR " + JSON.stringify(error));
                                _klogger.error(error);
                            }, existsOptions);
                        }
                    }
                };
                xhr.send();
            }
            else {
                try {
                    var data_1 = {
                        CHUNK_LENGTH: 10000,
                        startRange: 0,
                        isGetFileSize: false,
                        downloadedFileSize: 0,
                        dlResource: dlResource,
                        numberOfChunks: 0,
                        idx: 0,
                        totalFileSize: 0
                    };
                    if (dlResource.totalFileSize && dlResource.totalFileSize != NaN) {
                        data_1.totalFileSize = dlResource.totalFileSize;
                    }
                    data_1.numberOfChunks = Math.ceil(data_1.totalFileSize / data_1.CHUNK_LENGTH);
                    var options = { path: "" + ResourceManager.cacheRoot + dlResource.localFullDirectory };
                    if (dlResource.totalFileSize && dlResource.totalFileSize > 0) {
                        ResourceManager.storage.exists(function (res) {
                            if (res.exists != true) {
                                return ResourceManager.chunking(data_1);
                            }
                            else {
                                var removeOption = { file: "" + ResourceManager.cacheRoot + dlResource.localFullDirectory };
                                ResourceManager.storage.removeFile(function () {
                                    return ResourceManager.chunking(data_1);
                                }, function (error) {
                                    console.log("ERROR " + JSON.stringify(error));
                                    _klogger.error(error);
                                }, removeOption);
                            }
                        }, function (error) {
                            console.log("ERROR " + JSON.stringify(error));
                            _klogger.error(error);
                        }, options);
                    }
                    else {
                        ResourceManager.downloadNext();
                    }
                }
                catch (err) {
                    _klogger.error(err);
                    console.log(err);
                }
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.htmlChunker = function (input, idx) {
        var start = idx * 10000;
        var end = start + 10000;
        if (end > input.data.length) {
            end = input.data.length;
        }
        if (idx >= input.numberOfChunks) {
            SourcePatcher.baseUri = "..";
            return ResourceManager.postFileWork(input.path);
        }
        var dataChunk = new Uint8Array(input.data.slice(start, end));
        var options = {
            data: dataChunk,
            path: input.path,
            offset: 0,
            length: end - start,
            mode: 'append',
            encoding: 'binary',
            position: 0
        };
        return ResourceManager.storage.writeFile(function (res) {
            idx++;
            return ResourceManager.htmlChunker(input, idx);
        }, function (err) {
            var errorCode = err.errorCode;
            var errorText = err.errorText;
            console.log(" Error Code [" + errorCode + "]: " + errorText);
            _klogger.error(err);
        }, options);
    };
    ResourceManager.chunking = function (data) {
        data.startingRange = data.idx * data.CHUNK_LENGTH;
        if (data.startingRange + data.CHUNK_LENGTH > data.totalFileSize) {
            data.endingRange = data.totalFileSize;
        }
        else {
            data.endingRange = data.startingRange + data.CHUNK_LENGTH;
        }
        if (data.idx == data.numberOfChunks) {
            if (data.dlResource.fileName == 'weboscached.html') {
                SourcePatcher.baseUri = '.';
            }
            else if (data.dlResource.localFullDirectory.startsWith('/builder')) {
                SourcePatcher.baseUri = '../..';
            }
            else {
                SourcePatcher.baseUri = '..';
            }
            return ResourceManager.postFileWork("" + ResourceManager.cacheRoot + data.dlResource.localFullDirectory);
        }
        else {
            var xhr = new XMLHttpRequest();
            xhr.open('GET', data.dlResource.serverUrl);
            xhr.responseType = "arraybuffer";
            xhr.setRequestHeader('Range', "bytes=" + data.startingRange + "-" + data.endingRange);
            xhr.onreadystatechange = function () {
                if (xhr.readyState == 4) {
                    if (xhr.status === 200 || xhr.status === 206) {
                        data.idx++;
                        var buf = xhr.response;
                        data.responseBuffer = new Int8Array(buf);
                        var options = {
                            data: data.responseBuffer,
                            path: "" + ResourceManager.cacheRoot + data.dlResource.localFullDirectory,
                            mode: "append",
                            offset: 0,
                            length: data.CHUNK_LENGTH,
                            encoding: "binary"
                        };
                        return ResourceManager.storage.writeFile(function (cbObject) {
                            return ResourceManager.chunking(data);
                        }, function (err) {
                            var errorCode = err.errorCode;
                            var errorText = err.errorText;
                            console.log(" Error Code [" + errorCode + "]: " + errorText);
                            _klogger.error(err);
                        }, options);
                    }
                    else {
                        console.log(xhr.status);
                        _klogger.error(xhr.status);
                    }
                }
            };
            xhr.send();
        }
    };
    ResourceManager.downloadOnTimeout = function (dlResource) {
        return;
        // try {
        //     dlResource.tizenRequest = null;
        //     _klogger.info(`##### ResourceManager.onCompleted - DOWNLOAD TIMEOUT: ${dlResource.serverPath}`);
        //     tizen.download.cancel(dlResource.downloadId);
        //     clearTimeout(dlResource.dlTimeout);
        //     return ResourceManager.downloadNext();
        // } catch (err) {
        //     _klogger.error(err);
        // }
    };
    ResourceManager.postFileWorkDone = function (err) {
        try {
            if (err) {
                _klogger.error(err);
            }
            ResourceManager.downloadNext();
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.postFileWork = function (fullPath) {
        try {
            _klogger.debug("##### ResourceManager.postFileWork - " + fullPath);
            var ext = fullPath.substr(fullPath.lastIndexOf('.') + 1).toLowerCase();
            if (ext === 'html' || ext === 'css' || ext === 'js') {
                var name_1 = ResourceManager.getFilename(fullPath).toLowerCase();
                if (name_1 != 'playerengine.js') {
                    var fileSize_1 = 0;
                    var fileRead_1 = 0;
                    var file_1 = "";
                    return ResourceManager.storage.statFile(function (res) {
                        fileSize_1 = res.size;
                        return ResourceManager.readFile(fileSize_1, fullPath, fileRead_1, file_1);
                    }, function (err) {
                        console.log(err);
                        _klogger.error(err);
                    }, { path: fullPath });
                }
                else {
                    return ResourceManager.postFileWorkDone();
                }
            }
            else {
                return ResourceManager.postFileWorkDone();
            }
        }
        catch (err) {
            _klogger.error(err);
            return ResourceManager.postFileWorkDone(err);
        }
    };
    ResourceManager.readFile = function (fileSize, fullPath, fileRead, file) {
        var readLength;
        var position = fileRead;
        if (fileRead == fileSize) {
            var removeOptions = { file: fullPath };
            return ResourceManager.storage.removeFile(function () {
                var patchedFile = SourcePatcher.patch(file);
                var fileWritten = 0;
                return ResourceManager.writeFile(patchedFile.length, fullPath, fileWritten, patchedFile);
            }, function (err) {
                console.log(err);
                _klogger.error(err);
            }, removeOptions);
        }
        if (fileSize - fileRead > 10240) {
            readLength = 10240;
            fileRead += 10240;
        }
        else {
            readLength = fileSize - fileRead;
            fileRead += fileSize - fileRead;
        }
        var options = {
            path: fullPath,
            position: position,
            encoding: 'utf8',
            length: readLength
        };
        return ResourceManager.storage.readFile(function (res) {
            file += res.data;
            return ResourceManager.readFile(fileSize, fullPath, fileRead, file);
        }, function (error) {
            _klogger.error(error);
            return ResourceManager.postFileWorkDone(error);
        }, options);
    };
    ResourceManager.writeFile = function (fileSize, fullPath, fileWritten, file) {
        var writeLength;
        var position = fileWritten;
        if (fileWritten == fileSize) {
            return ResourceManager.storage.statFile(function (res) {
                return ResourceManager.postFileWorkDone();
            }, function (err) {
                console.log(err);
                _klogger.error(err);
            }, { path: fullPath });
        }
        if (fileSize - fileWritten > 10000) {
            writeLength = 10000;
            fileWritten += 10000;
        }
        else {
            writeLength = fileSize - fileWritten;
            fileWritten += fileSize - fileWritten;
        }
        if (file.slice(position, fileWritten).length == 0) {
            return ResourceManager.postFileWorkDone();
        }
        var options = {
            path: fullPath,
            data: file.slice(position, fileWritten),
            mode: 'append',
            length: writeLength,
            encoding: 'utf8',
        };
        return ResourceManager.storage.writeFile(function (res) {
            return ResourceManager.writeFile(fileSize, fullPath, fileWritten, file);
        }, function (err) {
            console.log(err);
            _klogger.error(err);
        }, options);
    };
    // static renameFile(sourcePath: string, fileName: string, callback: (fullPath: string) => any) {
    //     try {
    //         let srcDirectory = ResourceManager.getDirectory(sourcePath);
    //         let fullPath = srcDirectory + '/' + fileName;
    //         return tizen.filesystem.resolve(srcDirectory, (srcFile) => {
    //             return srcFile.moveTo(sourcePath, fullPath, true,
    //                 () => {
    //                     _klogger.info(`##### ResourceManager - FILE MOVED ${sourcePath} ==> ${fullPath}`);
    //                     return callback(fullPath);
    //                 },
    //                 (err) => {
    //                     _klogger.error(`##### ResourceManager - FILE MOVED FAILED ${sourcePath} ==> ${fullPath}\n${err}`);
    //                     return callback(fullPath);
    //                 });
    //         }, (err) => {
    //             _klogger.error(err);
    //             return callback(sourcePath);
    //         });
    //     } catch (err) {
    //         _klogger.error(err);
    //     }
    // }
    ResourceManager.getFileInfo = function (dlResource) {
        try {
            // DO ETAG LOGIC HERE TO CHECK AND SEE IF THE RESOURCE HAS BEEN UPDATED
            var reqType = void 0;
            if (dlResource.serverPath.startsWith('/embed') || dlResource.serverPath.startsWith('/message') || dlResource.serverPath.startsWith('/urgent')) {
                reqType = 'GET';
            }
            else {
                reqType = 'HEAD';
            }
            var xhr = new XMLHttpRequest();
            xhr.open(reqType, dlResource.serverUrl);
            xhr.onreadystatechange = function () {
                if (xhr.readyState == 4) {
                    if (xhr.status === 200 || xhr.status === 206) {
                        var etag = xhr.getResponseHeader('etag');
                        dlResource.totalFileSize = parseInt(xhr.getResponseHeader('Content-Length'));
                        if (localStorage.getItem(dlResource.localFullDirectory) === null || localStorage.getItem(dlResource.localFullDirectory) === "null" || localStorage.getItem(dlResource.localFullDirectory) !== etag) {
                            localStorage.setItem(dlResource.localFullDirectory, etag);
                            return ResourceManager.downloadCacheFile(dlResource);
                        }
                        else {
                            // CHECK IN CASE ETAG WAS SET BUT FILE DOES NOT EXIST
                            var options = { path: "" + ResourceManager.cacheRoot + dlResource.localFullDirectory };
                            ResourceManager.storage.exists(function (res) {
                                if (res.exists == true) {
                                    return ResourceManager.downloadNext();
                                }
                                else {
                                    return ResourceManager.downloadCacheFile(dlResource);
                                }
                            }, function (error) {
                                console.log("ERROR " + JSON.stringify(error));
                                _klogger.error(JSON.stringify(error));
                            }, options);
                        }
                    }
                    if (xhr.status === 404) {
                        ResourceManager.downloadNext();
                    }
                }
            };
            xhr.send();
        }
        catch (err) {
            _klogger.error(err);
            // return setTimeout(ResourceManager.downloadNext, 0);
            return ResourceManager.downloadNext();
        }
    };
    ResourceManager.createCacheDirectory = function (dlResource) {
        var options = { path: "" + ResourceManager.cacheRoot + dlResource.localSubDirectory };
        ResourceManager.storage.mkdir(function (data) {
            _klogger.debug('Content Cache Directory Created');
            return ResourceManager.getFileInfo(dlResource);
        }, function (error) {
            _klogger.error("Error Code: " + error.errorCode + "; with message: " + error.errorMessage);
            console.log("Error Code: " + error.errorCode + "; with message: " + error.errorMessage);
        }, options);
    };
    ResourceManager.checkCacheDirectory = function (dlResource) {
        // _klogger.debug(`##### ResourceManager - checkCacheDirectory(${JSON.stringify(dlResource)})`);
        try {
            var options = { path: "" + ResourceManager.cacheRoot + dlResource.localSubDirectory };
            ResourceManager.storage.exists(function (data) {
                if (!!data.exists) {
                    return ResourceManager.getFileInfo(dlResource);
                }
                else {
                    return ResourceManager.createCacheDirectory(dlResource);
                }
            }, function (error) {
                console.log("ERROR " + JSON.stringify(error));
                _klogger.error("ERROR " + JSON.stringify(error));
                return;
            }, options);
        }
        catch (err) {
            console.log('in the check cache directory catch');
            _klogger.error(err);
            console.log(err);
        }
    };
    ResourceManager.downloadNext = function () {
        try {
            if (!ResourceManager.dlPending || ResourceManager.dlPending.length < 1) {
                return ResourceManager.downloadComplete();
            }
            try {
                if (ResourceManager.dlPending.length > 0) {
                    var remaining = ResourceManager.resourceKeeper.length - ResourceManager.dlPending.length;
                    var percent = (((remaining) / ResourceManager.resourceKeeper.length) * 100).toFixed(2);
                    localStorage.setItem('KorbytDownloadProgress', remaining + "/" + ResourceManager.resourceKeeper.length + "(" + percent + "%)");
                    var dlResource = {
                        serverPath: ResourceManager.dlPending.pop(),
                    };
                    dlResource.serverUrl = main.pal.provisioning.cms + dlResource.serverPath;
                    if (dlResource.serverPath.endsWith('weboscached.html')) {
                        dlResource.localFullDirectory = dlResource.serverPath.slice(8);
                    }
                    else {
                        dlResource.localFullDirectory = dlResource.serverPath;
                    }
                    dlResource.localPath = ResourceManager.cacheRoot + dlResource.localFullDirectory;
                    dlResource.fileName = ResourceManager.getFilename(dlResource.localPath);
                    dlResource.localSubDirectory = ResourceManager.getDirectory(dlResource.localFullDirectory);
                    // _klogger.log(`Download Resource: ${JSON.stringify(dlResource)}`);
                    // start the download process...
                    return ResourceManager.checkCacheDirectory(dlResource);
                }
            }
            catch (err) {
                _klogger.error("##### ResourceManager - " + (err.stack || err.message));
            }
        }
        catch (err) {
            _klogger.error("##### ResourceManager - " + (err.stack || err.message));
        }
    };
    ResourceManager.cacheCleanDirectory = function (wipeYourCache, callback) {
        try {
            return callback();
            // if (!ResourceManager.cleanerDirectoryList || ResourceManager.cleanerDirectoryList.length < 1) {
            //     return ResourceManager.cacheDeleteFilesList(wipeYourCache, () => {
            //         _klogger.debug(`##### ResourceManager - completed cleanup of file cache`);
            //         return callback();
            //     });
            // }
            // let dir = ResourceManager.cleanerDirectoryList.pop();
            // return dir.listFiles((files) => {
            //     try {
            //         for (let ndx = 0; ndx < files.length; ndx++) {
            //             let file = files[ndx];
            //             if (file.isDirectory) {
            //                 // _klogger.debug(`ResourceManager.cleanerDirectoryList.push(${file.fullPath})`);
            //                 ResourceManager.cleanerDirectoryList.push(file);
            //             } else {
            //                 // _klogger.debug(`ResourceManager.cleanerFilesList.push(${file.fullPath})`);
            //                 ResourceManager.cleanerFilesList.push(file);
            //             }
            //         }
            //         ResourceManager.cacheCleanDirectory(wipeYourCache, callback);
            //     } catch (err) {
            //         _klogger.error(err);
            //         return callback();
            //     }
            // }, (err) => {
            //     _klogger.error(err);
            //     return callback();
            // });
        }
        catch (err) {
            _klogger.error(err);
            return callback();
        }
    };
    // static cacheDeleteFilesList(wipeYourCache: boolean, callback: () => any) {
    //     try {
    //         if (!ResourceManager.cleanerFilesList || ResourceManager.cleanerFilesList.length < 1) {
    //             return callback();
    //         }
    //         let file = ResourceManager.cleanerFilesList.pop();
    //         let path = file.fullPath.substring(baseDirectory.length);
    //         if (wipeYourCache || (ResourceManager.resourceKeeper.indexOf(path) < 0 && !file.fullPath.startsWith('downloads/prt/'))) {
    //             //if (wipeYourCache || ResourceManager.cacheDeleteCutoff.isAfter(file.modified)) 
    //             {
    //                 return ResourceManager.deleteDownloadedFile(file.fullPath, () => {
    //                     _klogger.debug(`##### ResourceManager ----------------------------------------------------------------------- DELETE ${file.fullPath}`);
    //                     return ResourceManager.cacheDeleteFilesList(wipeYourCache, callback);
    //                 });
    //             }
    //         }
    //         return setTimeout(() => {
    //             return ResourceManager.cacheDeleteFilesList(wipeYourCache, callback);
    //         }, 0);
    //     } catch (err) {
    //         _klogger.error(err);
    //         return callback();
    //     }
    // }
    ResourceManager.cacheCleaner = function (wipeYourCache, callback) {
        try {
            if (!callback) {
                callback = function () { };
            }
            var options = {
                device: 'internal'
            };
            ResourceManager.storage.removeAll(function (res) {
                console.log("Removed all Cache files: " + res);
                _klogger.log("Removed all Cache files: " + res);
                // localStorage.clear();
            }, function (error) {
                console.log("ERROR Faile to Remove Cache: " + error);
                _klogger.error("ERROR Faile to Remove Cache: " + error);
            }, options);
            return callback();
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    // private static startCacheCleaner() {
    //     if (ResourceManager.countFilesDownloaded < 1) {
    //         _klogger.debug(`##### ResourceManager - no files downloaded, skipping cache clean up pass.`);
    //     } else {
    //         _klogger.debug(`##### ResourceManager - starting wait timer for cache clean up`);
    //         clearTimeout(ResourceManager.cacheCleanTimer);
    //         ResourceManager.cacheCleanTimer = setTimeout(ResourceManager.cacheCleaner, 30 * 1000);
    //     }
    // }
    ResourceManager.downloadComplete = function () {
        try {
            console.log("CACHE DOWNLOAD COMPLETE");
            _klogger.info("CACHE DOWNLOAD COMPLETE");
            localStorage.setItem("Cached", "true");
            ResourceManager.isDownloading = false;
            localStorage.removeItem('KorbytDownloadProgress');
            if (!ResourceManager.isDownloading) {
                _klogger.error("ResourceManager.downloadComplete() - ResourceManager.isDownloading === " + ResourceManager.isDownloading);
            }
            ResourceManager.finishTime = new Date();
            var downloadTime = ResourceManager.finishTime.getTime() - ResourceManager.startTime.getTime();
            _klogger.info("##### ResourceManager - DOWNLOAD COMPLETED, downloaded " + ResourceManager.countFilesDownloaded + " of " + ResourceManager.resourceKeeper.length + " files in " + downloadTime / 1000 + " seconds");
            if (ResourceManager.finalCallback) {
                return ResourceManager.finalCallback(null);
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    ResourceManager.dlPending = [];
    ResourceManager.isDownloading = false;
    return ResourceManager;
}());
