var BlackoutSchedule = /** @class */ (function () {
    function BlackoutSchedule(_settings) {
        if (_settings === undefined) {
            _settings = {};
        }
        this.settings = {};
        this.settings.startTime = this.parseTime(_settings.blackout_start_time || '00:00');
        this.settings.endTime = this.parseTime(_settings.blackout_end_time || '00:00');
        this.settings.dow = this.parseDays(_settings.blackout_days || '');
        this.today = ''; // Force initialization
        this.lastCheck = false;
    }
    BlackoutSchedule.prototype.parseTime = function (ws) {
        var parts = ws.split(':');
        return (parseInt(parts[0], 10) * 60) + parseInt(parts[1], 10);
    };
    BlackoutSchedule.prototype.parseDays = function (ws) {
        var result = 0;
        for (var i = 0; i < ws.length; i++) {
            var day = parseInt(ws[i], 10);
            result |= 1 << day;
        }
        return result;
    };
    BlackoutSchedule.prototype.inBlackout = function () {
        var now = new Date();
        var dateString = now.toDateString();
        if (this.today != dateString) {
            this.today = dateString;
            this.blackout = [];
            for (var i = 0; i < 1440; i++) {
                this.blackout[i] = false; // Default to never blacked out
            }
            var dow = 1 << now.getDay();
            if (this.settings.dow & dow) {
                // Schedule applies to today
                var start = this.settings.startTime;
                var end = this.settings.endTime;
                if (end < start) {
                    end += 1440; // Ends past midnight wraps into the am
                }
                if (start !== end) { // When start and end are equal, never sleep
                    for (var m = start; m < end; m++) {
                        this.blackout[m % 1440] = true;
                    }
                }
            }
        }
        var minute = (now.getHours() * 60) + now.getMinutes();
        return this.blackout[minute];
    };
    BlackoutSchedule.prototype.leavingBlackout = function () {
        var currently = this.inBlackout();
        if ((currently === false) && (this.lastCheck === true)) {
            this.lastCheck = currently;
            return true;
        }
        this.lastCheck = currently;
        return false;
    };
    return BlackoutSchedule;
}());
