var iLogSeverity;
(function (iLogSeverity) {
    iLogSeverity[iLogSeverity["None"] = 0] = "None";
    iLogSeverity[iLogSeverity["Exception"] = 1] = "Exception";
    iLogSeverity[iLogSeverity["Error"] = 2] = "Error";
    iLogSeverity[iLogSeverity["Warn"] = 3] = "Warn";
    iLogSeverity[iLogSeverity["Info"] = 4] = "Info";
    iLogSeverity[iLogSeverity["Log"] = 5] = "Log";
    iLogSeverity[iLogSeverity["Debug"] = 6] = "Debug";
    iLogSeverity[iLogSeverity["Trace"] = 7] = "Trace";
    iLogSeverity[iLogSeverity["All"] = 8] = "All";
})(iLogSeverity || (iLogSeverity = {}));
var maximumLoggerSize = 256;
var Logger = /** @class */ (function () {
    function Logger(config) {
        this.messageStack = [];
        this.logLevel = iLogSeverity.Info;
        this.setWritter(this.logged);
        this.logDepth = maximumLoggerSize;
        if (config) {
            if (config.writter) {
                this.setWritter(config.writter);
            }
            if (config.depth) {
                this.setLogDepth(config.depth);
            }
            if (config.level) {
                this.setLogLevel(config.level);
            }
        }
    }
    Logger.prototype.setLogLevel = function (level) {
        this.logLevel = level;
    };
    Logger.prototype.setLogDepth = function (depth) {
        this.logDepth = depth;
    };
    Logger.postMessage = function (logged) {
        parent.postMessage(JSON.stringify({ to: 'host', logger: logged }), parent.location.toString());
    };
    Logger.postMessageSplat = function (logged) {
        parent.postMessage(JSON.stringify({ to: 'host', logger: logged }), '*');
    };
    Logger.prototype.setWritter = function (wrtr) {
        try {
            this.writter = wrtr;
            if (this.writter) {
                while (this.messageStack.length > 0) {
                    this.writter(this.messageStack.shift());
                }
            }
        }
        catch (err) {
            console.error(err);
        }
    };
    Logger.prototype.logged = function (log) {
        this.messageStack.push(log);
        if (this.messageStack.length > this.logDepth) {
            this.messageStack.shift();
        }
    };
    Logger.prototype.push = function (sev, msg) {
        var logged = { when: new Date().toISOString(), severity: sev, message: msg };
        this.writter(logged);
    };
    Logger.prototype.stringify = function () {
        var stuff = '\n';
        var _loop_1 = function () {
            var logged = this_1.messageStack.shift();
            // Note: cache should not be re-used by repeated calls to JSON.stringify.
            var cache = [];
            var txt = JSON.stringify(logged.message, function (key, value) {
                if (typeof value === 'object' && value !== null) {
                    if (cache.indexOf(value) !== -1) {
                        // Circular reference found, discard key
                        return;
                    }
                    // Store value in our collection
                    cache.push(value);
                }
                return value;
            });
            cache = null; // Enable garbage collection
            stuff += logged.when + " [" + iLogSeverity[logged.severity] + "] " + txt + "\n";
        };
        var this_1 = this;
        while (this.messageStack.length > 0) {
            _loop_1();
        }
        return stuff;
    };
    Logger.prototype.exception = function (message) {
        if (this.logLevel >= iLogSeverity.Exception)
            this.push(iLogSeverity.Exception, message);
    };
    Logger.prototype.error = function (message) {
        if (this.logLevel >= iLogSeverity.Error)
            this.push(iLogSeverity.Error, message);
    };
    Logger.prototype.warn = function (message) {
        if (this.logLevel >= iLogSeverity.Warn)
            this.push(iLogSeverity.Warn, message);
    };
    Logger.prototype.info = function (message) {
        if (this.logLevel >= iLogSeverity.Info)
            this.push(iLogSeverity.Info, message);
    };
    Logger.prototype.log = function (message) {
        if (this.logLevel >= iLogSeverity.Log)
            this.push(iLogSeverity.Log, message);
    };
    Logger.prototype.debug = function (message) {
        if (this.logLevel >= iLogSeverity.Debug)
            this.push(iLogSeverity.Debug, message);
    };
    Logger.prototype.trace = function (message) {
        if (this.logLevel >= iLogSeverity.Trace)
            this.push(iLogSeverity.Trace, message);
    };
    return Logger;
}());
