"use strict";
/// <reference path="../js/qrcode.js" />
function showProvisioningCode(token) {
    var options = {
        text: token,
        className: 'qrcode',
        id: 'qrcode',
        insertBefore: false
    };
    var el = document.getElementById(options.id);
    if (!el.firstChild) {
        make(el, options);
    }
}
function make(el, options) {
    try {
        var defaults = {
            render: "canvas",
            width: 256,
            height: 256,
            typeNumber: -1,
            correctLevel: QRErrorCorrectLevel.H,
            background: "#ffffff",
            foreground: "#000000",
            className: 'qrPanelWrapper',
            id: 'qrPanelWrapper',
            insertBefore: true
        };
        options = Object.assign(defaults, options);
        if (options.insertBefore) {
            el = document.createElement('div');
            el.className = options.className;
            el.id = options.id;
            document.body.insertBefore(el, document.body.firstChild);
        }
        var qrcode = new QRCode(options.typeNumber, options.correctLevel);
        qrcode.addData(options.text);
        qrcode.make();
        // create canvas element
        var canvas = document.createElement('canvas');
        canvas.width = options.width;
        canvas.height = options.height;
        var ctx = canvas.getContext('2d');
        // compute tileW/tileH based on options.width/options.height
        var tileW = options.width / qrcode.getModuleCount();
        var tileH = options.height / qrcode.getModuleCount();
        // draw in the canvas
        for (var row = 0; row < qrcode.getModuleCount(); row++) {
            for (var col = 0; col < qrcode.getModuleCount(); col++) {
                ctx.fillStyle = qrcode.isDark(row, col) ? options.foreground : options.background;
                var w = (Math.ceil((col + 1) * tileW) - Math.floor(col * tileW));
                var h = (Math.ceil((row + 1) * tileH) - Math.floor(row * tileH));
                ctx.fillRect(Math.round(col * tileW), Math.round(row * tileH), w, h);
            }
        }
        //var details = `<h1>Scan this QR Code</h1>`;
        if (options.insertBefore) {
            el.insertBefore(canvas, el.firstChild);
        }
        else {
            el.appendChild(canvas);
        }
    }
    catch (err) {
        _klogger.error(err);
    }
}
function updateDateTime() {
    var t = document.getElementById('localtime');
    if (t) {
        var now = moment();
        var ct = now.format("YYYY-MM-DD hh:mm:ss a");
        if (t.innerText != ct) {
            t.innerText = ct;
        }
        var nextSecond = 1000 - now.milliseconds();
        setTimeout(updateDateTime, nextSecond);
    }
}
;
function provisioningInit() {
    if (typeof Object.assign != 'function') {
        _klogger.info('========== polyfill: Object.assign');
        // Must be writable: true, enumerable: false, configurable: true
        Object.defineProperty(Object, "assign", {
            value: function assign(target, varArgs) {
                'use strict';
                if (target == null) { // TypeError if undefined or null
                    throw new TypeError('Cannot convert undefined or null to object');
                }
                var to = Object(target);
                for (var index = 1; index < arguments.length; index++) {
                    var nextSource = arguments[index];
                    if (nextSource != null) { // Skip over if undefined or null
                        for (var nextKey in nextSource) {
                            // Avoid bugs when hasOwnProperty is shadowed
                            if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
                                to[nextKey] = nextSource[nextKey];
                            }
                        }
                    }
                }
                return to;
            },
            writable: true,
            configurable: true
        });
    }
    if (PlayerHost && !PlayerHost.pal) {
        PlayerHost.init();
    }
    if (!PlayerHost || !PlayerHost.pal || !PlayerHost.pal.initialize) {
        console.debug("??? NOT READY YET ???");
        return setTimeout(provisioningInit, 50);
    }
    PlayerHost.pal.initialize(function () {
        PlayerHost.pal.provisioningDetails(function (list) {
            var details = "";
            for (var d = 0; d < list.length; d++) {
                if (list[d].v) {
                    details += "<div class=\"row\"><label>" + list[d].n + "</label><span>" + list[d].v + "</span></div>";
                }
            }
            details += "<div class=\"row\"><label>Local Time</label><span id=\"localtime\">pending</span></div>";
            //console.info(details);
            $('#playerData').html(details);
            setTimeout(updateDateTime, 100); // first update after 1/10 of a second
            $('#token').html(PlayerHost.pal.provisioningToken);
            showProvisioningCode('pt=' + PlayerHost.pal.provisioningToken);
            $('#wsUrl').val(PlayerHost.pal.wsUrl);
            $('#save').click(function (e) {
                e.preventDefault();
                PlayerHost.pal.wsUrl = $('#wsUrl').val();
                PlayerHost.pal.saveSettings(function (err) {
                    if (err)
                        _klogger.error(err);
                    PlayerHost.pal.sendMessage('restart');
                });
            });
        });
    });
}
