"use strict";
var QRPanel = /** @class */ (function () {
    function QRPanel() {
    }
    QRPanel.show = function (data) {
        try {
            var existing = document.getElementById('qrPanelWrapper');
            if (!existing) {
                var options = {
                    text: "player_name:" + (data.playerName || data.player_name) + ",player_id:" + (data.playerID || data.player_id) + ",tenant_id:" + (data.tenantID || data.tenant_id)
                };
                QRPanel.make(QRPanel.overWrap, options);
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    QRPanel.hide = function () {
        try {
            var existing = document.getElementById('qrPanelWrapper');
            if (existing) {
                existing.remove();
            }
        }
        catch (err) {
            _klogger.error(err);
        }
    };
    QRPanel.showProvisioningCode = function (token) {
        var options = {
            text: token,
            className: 'qrcode',
            id: 'qrcode',
            insertBefore: false
        };
        var el = document.getElementById(options.id);
        if (!el.firstChild) {
            QRPanel.make(el, options);
        }
    };
    QRPanel.make = function (el, options) {
        var defaults = {
            render: "canvas",
            width: 256,
            height: 256,
            typeNumber: -1,
            correctLevel: QRErrorCorrectLevel.H,
            background: "#ffffff",
            foreground: "#000000",
            className: 'qrPanelWrapper',
            id: 'qrPanelWrapper',
            insertBefore: true
        };
        options = Object.assign(defaults, options);
        if (options.insertBefore) {
            el = document.createElement('div');
            el.className = options.className;
            el.id = options.id;
            document.body.insertBefore(el, document.body.firstChild);
        }
        var qrcode = new QRCode(options.typeNumber, options.correctLevel);
        qrcode.addData(options.text);
        qrcode.make();
        // create canvas element
        var canvas = document.createElement('canvas');
        canvas.width = options.width;
        canvas.height = options.height;
        var ctx = canvas.getContext('2d');
        // compute tileW/tileH based on options.width/options.height
        var tileW = options.width / qrcode.getModuleCount();
        var tileH = options.height / qrcode.getModuleCount();
        // draw in the canvas
        for (var row = 0; row < qrcode.getModuleCount(); row++) {
            for (var col = 0; col < qrcode.getModuleCount(); col++) {
                ctx.fillStyle = qrcode.isDark(row, col) ? options.foreground : options.background;
                var w = (Math.ceil((col + 1) * tileW) - Math.floor(col * tileW));
                var h = (Math.ceil((row + 1) * tileH) - Math.floor(row * tileH));
                ctx.fillRect(Math.round(col * tileW), Math.round(row * tileH), w, h);
            }
        }
        //var details = `<h1>Scan this QR Code</h1>`;
        if (options.insertBefore) {
            el.insertBefore(canvas, el.firstChild);
        }
        else {
            el.appendChild(canvas);
        }
    };
    return QRPanel;
}());
