var LogSeverity;
(function (LogSeverity) {
    LogSeverity[LogSeverity["None"] = 0] = "None";
    LogSeverity[LogSeverity["Exception"] = 1] = "Exception";
    LogSeverity[LogSeverity["Error"] = 2] = "Error";
    LogSeverity[LogSeverity["Warn"] = 3] = "Warn";
    LogSeverity[LogSeverity["Info"] = 4] = "Info";
    LogSeverity[LogSeverity["Log"] = 5] = "Log";
    LogSeverity[LogSeverity["Debug"] = 6] = "Debug";
    LogSeverity[LogSeverity["Trace"] = 7] = "Trace";
    LogSeverity[LogSeverity["All"] = 8] = "All";
})(LogSeverity || (LogSeverity = {}));
var maxLogSize = 256;
var hasProp = Object.prototype.hasOwnProperty;
function throwsMessage(err) {
    return '[Throws: ' + (err ? err.message : '?') + ']';
}
function safeGetValueFromPropertyOnObject(obj, property) {
    if (hasProp.call(obj, property)) {
        try {
            return obj[property];
        }
        catch (err) {
            return throwsMessage(err);
        }
    }
    return obj[property];
}
function ensureProperties(obj) {
    var seen = []; // store references to objects we have seen before
    function visit(obj) {
        if (obj === null || typeof obj !== 'object') {
            return obj;
        }
        if (seen.indexOf(obj) !== -1) {
            return '[Circular]';
        }
        seen.push(obj);
        if (typeof obj.toJSON === 'function') {
            try {
                return visit(obj.toJSON());
            }
            catch (err) {
                return throwsMessage(err);
            }
        }
        if (Array.isArray(obj)) {
            return obj.map(visit);
        }
        return Object.keys(obj).reduce(function (result, prop) {
            // prevent faulty defined getter properties
            result[prop] = visit(safeGetValueFromPropertyOnObject(obj, prop));
            return result;
        }, {});
    }
    ;
    return visit(obj);
}
var VisibleLogger = /** @class */ (function () {
    function VisibleLogger() {
    }
    VisibleLogger.insertConsoleMessage = function (severity, args, color) {
        var timeStampInMs = new Date().toTimeString().substr(0, 8);
        var text = '';
        args.forEach(function (arg) {
            if (arg instanceof Error) {
                text += arg.stack;
            }
            else if (typeof (arg) !== 'string') {
                text += JSON.stringify(ensureProperties(arg), null, 2);
            }
            else {
                text += ' ' + arg;
            }
        });
        VisibleLogger.logged({ when: timeStampInMs, severity: severity, message: text });
        if (VisibleLogger.visible) {
            var d = document.createElement('div');
            var s = document.createElement('span');
            s.innerText = timeStampInMs + "|" + String("   " + LogSeverity[severity]).slice(-5) + "|" + text;
            d.appendChild(s);
            s.id = 'debugMessage';
            s.style.color = color;
            VisibleLogger.dbgWrap.appendChild(d);
            VisibleLogger.dbgWrap.scrollTop = VisibleLogger.dbgWrap.scrollHeight;
            if (VisibleLogger.dbgWrap.childNodes.length > 66) {
                VisibleLogger.dbgWrap.removeChild(VisibleLogger.dbgWrap.childNodes[0]);
            }
        }
    };
    VisibleLogger.show = function (mode) {
        VisibleLogger.visible = mode;
        var existing = document.getElementById('debugWrapper');
        if (mode) {
            if (!existing) {
                VisibleLogger.dbgWrap = document.createElement('div');
                VisibleLogger.dbgWrap.className = 'debugWrapper';
                VisibleLogger.dbgWrap.id = 'debugWrapper';
                document.body.insertBefore(VisibleLogger.dbgWrap, document.body.firstChild);
            }
            else {
                if (existing) {
                    existing.remove();
                }
            }
        }
    };
    VisibleLogger.setLogLevel = function (level) {
        VisibleLogger.logLevel = level;
    };
    VisibleLogger.setLogDepth = function (depth) {
        VisibleLogger.logDepth = depth;
    };
    VisibleLogger.logged = function (log) {
        VisibleLogger.messageStack.push(log);
        if (VisibleLogger.messageStack.length > this.logDepth) {
            VisibleLogger.messageStack.shift();
        }
    };
    VisibleLogger.stringify = function () {
        var stuff = '\n';
        VisibleLogger.messageStack.forEach(function (logged) {
            stuff += logged.when + " [" + LogSeverity[logged.severity] + "] " + logged.message + "\n";
        });
        return stuff;
    };
    VisibleLogger.installHook = function (oldCons) {
        try {
            if (VisibleLogger.hook) {
                return VisibleLogger.hook;
            }
            VisibleLogger.messageStack = [];
            VisibleLogger.logLevel = LogSeverity.Info;
            VisibleLogger.logDepth = maxLogSize;
            VisibleLogger.hook = {
                debug: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    oldCons.debug.apply(oldCons, args);
                    VisibleLogger.insertConsoleMessage(LogSeverity.Debug, args, "AQUA");
                },
                log: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    oldCons.log.apply(oldCons, args);
                    VisibleLogger.insertConsoleMessage(LogSeverity.Log, args, "LIME");
                },
                info: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    oldCons.info.apply(oldCons, args);
                    VisibleLogger.insertConsoleMessage(LogSeverity.Info, args, "WHITE");
                },
                warn: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    oldCons.warn.apply(oldCons, args);
                    VisibleLogger.insertConsoleMessage(LogSeverity.Warn, args, "OLIVE");
                },
                error: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    oldCons.error.apply(oldCons, args);
                    VisibleLogger.insertConsoleMessage(LogSeverity.Error, args, "RED");
                },
                clog: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    oldCons.info.apply(oldCons, args);
                    VisibleLogger.insertConsoleMessage(LogSeverity.Info, args, "WHITE");
                },
                assert: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    oldCons.assert.apply(oldCons, args);
                }
            };
            return VisibleLogger.hook;
        }
        catch (err) {
            oldCons.error(err.stack || err);
        }
    };
    VisibleLogger.hook = undefined;
    return VisibleLogger;
}());
